/* DTEMP Digitaler Khlschrankregler mit TMP04 und LCD              */
/*                                                                  */
/*                       __    _                                    */ 
/*            __________/ /_  (_)________   ___  __  __             */ 
/*           / ___/ ___/ __ \/ / ___/ __ \ / _ \/ / / /             */ 
/*          (__  ) /__/ / / / / /  / /_/ //  __/ /_/ /              */ 
/*         /____/\___/_/ /_/_/_/  / .___(_)___/\__,_/               */ 
/*                               /_/                                */ 
/*                                                                  */ 
/* (c) C.Schirp  released for non-commercial use                    */ 
/*     this header must be distributed with this source code        */ 
/*                                                                  */
/* Division 24/16 (c) by Nikolai Golovchenko                        */
/* bin2dec999 (c) by Nikolai Golovchenko                            */
/* mul16 (c) by Scott Dattalo                                       */
/* LCD-Routinen (c) Holger Klabunde mit Anpassungen von mir         */
/*                                                                  */
/* V1.0 Erstausgabe                                                 */
/* V1.1 Int-Fehlerhandler, Fehler-LED                               */
/* V1.2 Medianfilter fr Temperaturen, Fehlerbits angepasst         */
/* V1.3 selbstdefinierte Zeichen, mplab wg. 1k-Grenze               */
/* V1.4 Fehlerbehandlung Sensorfehler verbessert,                   */
/*      Kommentare erw., Einschaltmeldung                           */
/*                                                                  */


#include <c:\projekte\picsel\cc5\16F628.h>
#include <c:\projekte\picsel\cc5\int16CXX.h>
#include <c:\projekte\picsel\cc5\inline.h>
#include <c:\projekte\picsel\dtemp_2\dtemp.h>

#pragma config =0x3F38 

// Standard ohne ext reset 0x3F18
// Standard mit ext Reset  0x3F38  

/* Port- Defines */

#define LCD_RS    0b.00010000             //RS-Pin ist Bit4, LCD Register Select
#define LCD_CS    0b.01000000             //E-Pin ist Bit6, LCD Chip Select

#define KOMP_RELAIS RB3
#define LED_R   RB4
#define LED_G   RB5

#define TMP_1   RB0
#define TMP_2	RB1
#define DOOR_OPEN RB2

#pragma char PORTQ @ PORTA               //PORT auswhlen, DB7-Bit7, DB6-Bit6, DB5-Bit5, DB4-Bit4

#define LCDCS_ON() { NOP;  NOP; lcdport|=LCD_CS; PORTQ=lcdport; }
#define LCDCS_OFF() { NOP;  NOP; lcdport&=~LCD_CS; PORTQ=lcdport; }
#define LCDRS_ON() { NOP;  NOP; lcdport|=LCD_RS; PORTQ=lcdport; }
#define LCDRS_OFF() { NOP; NOP; lcdport&=~LCD_RS; PORTQ=lcdport; }

#define Wait() Delay1ms(10)

void Komp_Aus(void);


//Kommandos fr das LCD-Display
#define CLEAR_DISPLAY 0x01
#define OFF_DISPLAY   0x08
#define CURSOR_OFF    0x0C
#define CURSOR_ON     0x0E
#define CURSOR_BLINK  0x0F
#define SHIFT_LEFT    0x18
#define SHIFT_RIGHT   0x1C

/* Variablen */

uns8 lcdport;

bank0 int16 Temperatur;	// berechnete Temperatur (Zwischenspeicher)
bank0 uns16 t_low;	// gemessene Zeit LOW
bank0 uns16 t_high;	// gemessene Zeit HI
bank0 int16 temp_G;	// Temperatur Gefrierfach signed, gefiltert
bank0 int16 temp_K;	// Temperatur Kuehlfach signed, gefiltert
bank0 uns16 Komp_OnTime; // Zeit in s, die der Kompressor schon luft
bank0 uns16 Komp_prtkt;	// verbleibende Schutzzeit bis wiederanlauf

unsigned char Text[9];	// Speicher fr eine LCD-Zeile incl Nullterminierung

bank0 uns8 Komp_ON:1;	// Flag fr Kompressor eingeschaltet
bank0 uns8 F_SensorG:1;	// Flag fr Sensorfehler TMP1 / Gefrierfach
bank0 uns8 F_SensorK:1; // Flag fr Sensorfehler TMP2 / Kuehlfach
bank0 uns8 F_MotorMax:1;// Flag fr Motor-Maxlaufzeit
bank0 uns8 Komp_Req:1;	// Flag fr "kompressor erforderlich"
bank0 uns8 F_Frost:1;	// Falg fr Kompressor-ausschalten wg. Frostschutz
bank0 uns8 VZ_merker:1;	// Vorzeichenmerker und Hilfsbit
bank0 uns8 Blink:1;	// Bitcounter fr Alive-Anzeige im LCD

uns8 q[4];	// Ergebnisvariable fr mul16by16

int16 med_a;
int16 med_b;
int16 med_c;

// Divisor und Dividend fr div24by16
//uns8 AARG[3];
//uns16 BARG;


/* defines fuer die SW */

#define Hund Text+1
#define Tens Text+2
#define Ones Text+4

//	AARG q[2]
//	AARG+1 q[1]
//	AARG+2 q[0]
		
//#define	BARG.low8 =t_low.high8;
//#define	BARG.high8=t_low.low8;

#define KOMP_PROT 60	// 60 10er Sekunden Schutzzeit = 10Minuten
#define KOMP_MAXRUN 300  // 300 10er Sekunden Maxlaufzeit = 50 Minuten
#define G_TEMP_MAX ((int16)-5*17)	
#define G_TEMP_MIN ((int16)-5*22) 
#define K_TEMP_MAX ((int16)5*8) 
#define K_TEMP_MIN ((int16)32) 	// 6.4Grad
#define K_FROST_MIN ((int16)5*3) 
#define K_CHECK_MAX ((int16)5*40) // 40
#define K_CHECK_MIN ((int16)-5*35) // -35
#define K_FAIL_TEMP_K ((int16)5*6)
#define K_FAIL_TEMP_G ((int16)-5*23)



#define CGRAM_LEN	24  //Anzahl zu bertragender Zeichen

const unsigned char chardefs[] = {
//Adresse 0 im Character RAM
// 'raute'
0, 0x04, 0x0A, 0x11, 0x0A, 0x04, 0, 0,

//Adresse 1 im Character RAM
// 'quadrat'
0, 0, 0x0e, 0x0a, 0x0e, 0, 0, 0,

//Adresse 2 im Character RAM
// 'gefuelltes quadrat'
0, 0, 0x0e, 0x0e, 0x0e, 0, 0, 0,

};


/*------------------------------------------------------------------*/
/*   II  N   N TTTTT                                                */
/*   II  NN  N   T                                                  */
/*   II  N N N   T                                                  */
/*   II  N  NN   T                                                  */
/*   II  N   N   T                                                  */
/*------------------------------------------------------------------*/
#pragma origin 4
interrupt int_server(void)
{
	int_save_registers
	
	// Eigentlich sind alle Ints gesprerrt - daher hier nur Fehlerled setzen
	
	LED_R = 0;
	
	int_restore_registers
}



/*------------------------------------------------------------------*/
/*   II  N   N  II TTTTT                                            */
/*   II  NN  N  II   T                                              */
/*   II  N N N  II   T                                              */
/*   II  N  NN  II   T                                              */
/*   II  N   N  II   T                                              */
/*------------------------------------------------------------------*/

void init (void)
{
	/* PORT A, B Config */
	CMCON = 0x07;		// kein Komparator
	
	TRISB = 0b0000.0011; 	//Alles Ausgnge, 0,1 Eingaenge	
	TRISA = 0b0010.0000;	// PA output, ra5 input
	
	PORTB = 0b0011.0000;	// HINT: erst Direction, dann Wert!
	PORTA = 0x00;	

	/*sonstiges*/
	T1CON = 0x00;		// Timer disabled
	OPTION_REG = 0x04;	/* pull-up portB, falling edge int, no WDT prescaler, prescale=32 */
	INTCON = 0; 		// reset and disable all interrupts by default

	/* Variablen initialisieren */

	lcdport=0x80;	// alle low, nur bit 7 (VDD) auf 1
	
	Text[5] =' ';	// Spaces im Text...
	Text[6] =' ';
	Text[7] =' ';
	
	Text[8] = 0;	// Nullterminierung fr LCD-String
	
	temp_G = 0;	// Temperaturfilter initialisieren
	temp_K = 0;
	
 	F_SensorG=0;	// Fehlerflags loeschen
	F_SensorK=0;
	F_MotorMax=0;
	
	Komp_Aus();

	
}

/*------------------------------------------------------------------*/
/*   LL    CCCC DDD                                                 */
/*   LL   CC    DD DD                                               */
/*   LL   CC    DD  DD                                              */
/*   LL   CC    DD DD                                               */
/*   LLLLL CCCC DDD                                                 */
/*------------------------------------------------------------------*/

void Delay1ms(uns16 ms)
{

	while(ms)				// Schleife verlassen wen ms=0 ist
	{
	    OPTION = 2; 		// Vorteiler auf 8 einstellen
	    TMR0 = 131; 		// 125 * 8 = 1000 (= 1 ms)
	    while (TMR0);		// abwarten einer Milisekunde
	    ms--;				// "ms" mit jeder Milisekunde ernidrigen
	}

}

//*******************************************
// Schreibt ein Datenbyte in die LCD-Anzeige
// Schreibt das Byte in zwei 4 Bit Nibbles
//*******************************************

void LCDWriteByte (unsigned char data)
{
 unsigned char buff;
 lcdport= PORTQ; 		//alten Wert merken
 LCDCS_OFF();
 buff=data>>4;   		//High Nibble des Bytes
 buff&=0x0F;			//Rest lschen
 lcdport=lcdport&0xF0;		//Hier D0-D3 lschen zum
 lcdport|=buff;			//reinodern vom High Nibble
 lcdport|=LCD_CS;
 LCDCS_ON();	      		//High Nibble schreiben

 LCDCS_OFF();
 buff=data&0x0F;        	//Low Nibble des Bytes
 lcdport=lcdport&0xF0;		//D0-D3  lschen zum
 lcdport|=buff;			//reinodern vom Low Nibble
 LCDCS_ON();	      		//Low Nibble schreiben
 LCDCS_OFF();
 Delay1ms(1);       		// warten
}

//*****************************************
// Commando ans Display senden
//*****************************************
void LCDCommand(unsigned char mode)
{
 LCDRS_OFF(); 			//Controlregister
 LCDWriteByte(mode);
 LCDRS_ON(); 			//Zurck auf Datenregister
}

//*****************************************
// Lscht die LCD Anzeige
// Cursor steht dann auf Position 1,Zeile 1
// Gilt auch fr mehrzeilige Anzeigen
//*****************************************
void LCDCls(void)
{
 LCDRS_OFF(); //Controlregister
 LCDWriteByte(0x01); //CLS
 LCDRS_ON(); //Umschalten auf Datenregister
 Wait();
}
//**************************************************
//Schreibt einen Text in die LCD-Anzeige
//**************************************************
void LCDString(const char *__putstr)
{
#pragma updateBank 0 /* OFF, all cores */
 char __ps;

 __ps = *__putstr;

  while(__ps>0)
   {
    __putstr++;
    if (__ps== 0) break;
   LCDWriteByte(__ps);
    __ps = *__putstr;
   }
#pragma updateBank 1 /* ON, all cores */
}
//****************************************************
// Springt auf Zeile x und Position y der LCD-Anzeige
// Gltig fuer 2 Zeilen : 0,1.
//****************************************************
void LCDPos(unsigned char zeile,unsigned char position)
{
#pragma updateBank 0 /* OFF, all cores */
	 unsigned char buff;
	
	// 0x80 wegen Display Adress Set.
	// Erste Position ist bei mir  0.
	
	if (zeile)
	{
		buff=0xC0+position;
	}
	else
	{
		buff=0x80+position;
	}
	
	 LCDCommand(buff);
#pragma updateBank 1 /* ON, all cores */
}


//*********************************************
//Stellt die Funktionsweise der LCD Anzeige ein
//*********************************************

void LCDInit(void)
{
 int i;

 LCDRS_OFF();  //Control Register
 LCDCS_OFF();  //LCD-Anzeige abwhlen

//Software Reset laut Hitachi Datenblatt
 Delay1ms(60); //Mglichen Power On Reset abwarten
 for(i=0; i<3; i++)
  {
   lcdport=lcdport&0xF0;//D0-D3 lschen
   lcdport|=0x03;	//Wert reinodern
   PORTQ=lcdport;	//Auf den Port legen
   LCDCS_OFF(); //LCD-Anzeige abwhlen
   NOP;
   LCDCS_ON();	//High Nibble schreiben
   Delay1ms(5); //Ein bichen warten auf die Anzeige
  } 
//Ende Software Reset


//Der erste Schreibzugriff nach dem Reset ist ein 8 Bit
//Zugriff!! Also nur einmal schreiben, nicht Low-High Nibble.

//Um die 4Bit Operation einzuschalten
 lcdport=0x02;	//4 Bit Operation	

 PORTQ=lcdport;	//Auf den Port legen
 NOP;

 LCDCS_ON();	//High Nibble schreiben
 NOP;
 NOP; 
 LCDCS_OFF();
 Wait();
 LCDWriteByte(0x2C);    //4 Bit Operation,2 Line,5x7 Font  5
 Wait();
 LCDWriteByte(0x14); 	//Cursor Move,Right Shift
 Wait();

 LCDWriteByte(0x0C); 	//Display on, Cursor off ,Cursor no blink
 Wait();

 LCDWriteByte(0x06); 	//Increment, Display Freeze
 Wait();
 LCDWriteByte(0x02); 	//Cursor Home
 Wait();
 LCDRS_ON();		//LCD-Datenregister

 LCDCommand(CLEAR_DISPLAY);
 Wait();
 
//Die eigenen Character ins CGRAM schreiben

//Der interne Adresszhler des LCD-Displays luft
//nach jedem Schreibzugriff automatisch hoch

 LCDCommand(0x48);//CG_RAM Adresse(1) setzen

 for(i=0; i<CGRAM_LEN; i++) //8 Codes pro Zeichen bertragen
  {
   LCDWriteByte(chardefs[i]);
  }


 LCDCommand(0x80); //DD_RAM Adresse 0 setzen
}

#ifdef _DEBUG
/*------------------------------------------------------------------*/
/*  Konvertiert 16 Bit Binaer zu Hexadezimal-ASCII 4-stellig        */
/*------------------------------------------------------------------*/
void Dez4Hex(uns16 zahl)
{
	char  count, help;

	for (count = 0; count < 4; count++)
	{
		
		help = zahl & 0x000F;
		
		if (help > 9)
		{
			Text[3 - count] = help + 0x37;
		}
		else
		{
			Text[3 - count] = help + 0x30;
		}
		zahl >>=4;
	}
}
#endif

/*------------------------------------------------------------------*/
/*  Konvertiert 16 Bit Binaer zu Dezimal-ASCII 3-stellig            */
/*------------------------------------------------------------------*/
void bin2dec999(uns16 Num)
{
#asm
;Binary to decimal conversion (0..999)
;
;Input: Num+1:Num
;Output Hund:Tens:Ones (-> per define auf Text[])
;
;If Input > 999 Output will roll over, e.g.
;for input=5678 output=678.
;
;
;Size: 34 instructions
;Execution  time  (max) including return:
;22+5*9-1+5*6-1+4*3-1+2 = 108 cycles
;
;5-July-2000 by Nikolai Golovchenko


        movf Num+1, w
        addlw 241
        addwf Num+1, w
        movwf Hund      ;b_2 = 2a_2 - 15

        addwf Hund, w
        addwf Hund, w
        addlw 253
        movwf Tens
        swapf Num, w
        andlw 0x0F
        addwf Tens, f
        addwf Tens, f   ;b_1 = 6a_2 + 2a_1 - 48

        addwf Num+1, w
        sublw 251
        movwf Ones
        addwf Ones, f
        addwf Ones, f
        addwf Ones, f
        movf Num, w
        andlw 0x0F
        addwf Ones, f   ;b_0 = a_0 - 4(a_2 + a_1) - 20

        movlw 10
bin2dec999a                     ;9 cycles max
        addwf Ones, f
        decf Tens, f
        btfss 3,Carry
         goto bin2dec999a

bin2dec999b                     ;6 cycles max
        addwf Tens, f
        decf Hund, f
        btfss 3,Carry
         goto bin2dec999b

bin2dec999c                     ;3 cycles max
        addwf Hund, f
        btfss 3,Carry
         goto bin2dec999c

#endasm
	Text[1] +=0x30;
	Text[2] +=0x30;
	Text[3] ='.';
	Text[4] +=0x30;
}




/*------------------------------------------------------------------*/
/*   MM     MM    AA  TTTTTT HH   HH                                */
/*   MMM   MMM  AA  AA  TT   HH   HH                                */
/*   MM M M MM  AAAAAA  TT   HHHHHHH                                */
/*   MM  M  MM  AA  AA  TT   HH   HH                                */
/*   Mm     MM  AA  AA  TT   HH   HH                                */
/*------------------------------------------------------------------*/



void mul16(uns16 n1, uns16 n2)
{

#asm
;*******************************************************************
; Double Precision Multiplication
;
; n_2 : n_1 * n_4 : n_3 -> q_4:q_3:q_2:q_1
;
; Standard shift and add.
; Execution time: 215 to 295 clock cycles.
; Code space: 22 locations
;
; Cleaned up and corrected version from Microchip Ap note by BF.
; Note: Ap note has errors! Additional mods by Scott Dattalo.
;
;*******************************************************************

	clrf q+3
	clrf q+2
	clrf q+1
	clrf q	
	bsf q+1, 7	
m1:
	rrf n1+1, f
	rrf n1, f
	btfss 3,Carry  ; skpc
	goto m2
	movf n2, w
	addwf q+2, f
	movf n2+1, w
	btfsc 3,Carry  ; skpnc
	incfsz n2+1, w
	addwf q+3, f
m2:
	rrf q+3, f
	rrf q+2, f
	rrf q+1, f
	rrf q, f
	btfss   3,Carry   ;skpc
	goto m1
#endasm

}
        
        
void divide()
{       
        
	uns16 REM;  	// Divisionsrest -> Discard
	uns8 LOOPCOUNT;

#asm    
        
;Inputs:
;   Dividend - AARGB0:AARGB1:AARGB2 (0 - most significant!)
;   Divisor  - BARGB0:BARGB1
;Temporary:
;   Counter  - LOOPCOUNT
;   Remainder- REMB0:REMB1
;Output:
;   Quotient - AARGB0:AARGB1:AARGB2	  
        
FXD2416U:
            clrf REM
            clrf REM+1
            movlw .24
            movwf LOOPCOUNT
LOOPU2416:
            rlf q, W           ;shift dividend left to move next bit to remainder
            rlf q+1, F           ;
            rlf q+2, F           ;
        
            rlf REM+1, F            ;shift carry (next dividend bit) into remainder
            rlf REM, F
        
            rlf q, F           ;finish shifting the dividend and save  carry in AARGB2.0,
                                    ;since remainder can be 17 bit long in some cases
                                    ;(e.g. 0x800000/0xFFFF). This bit will also serve
                                    ;as the next result bit.
             
            movf t_low, W          ;substract divisor from 16-bit remainder
            subwf REM+1, F          ;
            movf t_low+1, W          ;
            btfss STATUS, Carry         ;
            incfsz t_low+1, W        ;
            subwf REM, F          ;

    ;here we also need to take into account the 17th bit of remainder, which
    ;is in AARGB2.0. If we don't have a borrow after subtracting from lower
    ;16 bits of remainder, then there is no borrow regardless of 17th bit 
    ;value. But, if we have the borrow, then that will depend on 17th bit 
    ;value. If it is 1, then no final borrow will occur. If it is 0, borrow
    ;will occur. These values match the borrow flag polarity.

            btfsc 3,Carry           ;skpnc - if no borrow after 16 bit subtraction
             bsf q, 0          ;then there is no borrow in result. Overwrite
                                    ;AARGB2.0 with 1 to indicate no
                                    ;borrow.
                                    ;if borrow did occur, AARGB2.0 already
                                    ;holds the final borrow value (0-borrow,
                                    ;1-no borrow)

            btfsc q, 0         ;if no borrow after 17-bit subtraction
             goto UOK46LL           ;skip remainder restoration.

            addwf REM, F          ;restore higher byte of remainder. (w 
                                    ;contains the value subtracted from it
                                    ;previously)
            movf t_low, W          ;restore lower byte of remainder
            addwf REM+1, F          ;

UOK46LL:

            decfsz LOOPCOUNT, f     ;decrement counter
             goto LOOPU2416         ;and repeat the loop if not zero.

 #endasm         
            
}

/*------------------------------------------------------------------*/
/*     AA    PPPP   PPPP                                            */
/*   AA  AA  PP PP  PP PP                                           */
/*   AAAAAA  PPPP   PPPP                                            */
/*   AA  AA  PP     PP                                              */
/*   AA  AA  PP     PP                                             */
/*------------------------------------------------------------------*/


// #######################################
//  Temp1 / Gefrierfach messen
// #######################################
void Measure_G(void)
{
	T1CON=0b0011.0000;  // Timer1 int, div8, stop
	TMR1H=0;
	TMR1L=0;
	TMR1IF=0;
	//T1CON=0b0011.0001;  // Timer1 int, div8, run
	TMR1ON=1;
#pragma updateBank 0 /* OFF, all cores */

	while ((!TMP_1)&&(!TMR1IF))//&&(t_low)
	{
	};
	//T1CON=0b0011.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_low.low8 = TMR1L;
	t_low.high8 = TMR1H;
	
	TMR1H=0;
	TMR1L=0;
	//T1CON=0b0011.0001;  // Timer1 int, div8, run
	TMR1ON=1;
	while ((TMP_1)&&(!TMR1IF))//&&(t_high)
	{
	};
	//T1CON=0b0010.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_high.low8 = TMR1L;
	t_high.high8 = TMR1H;
	
	
	TMR1H=0;
	TMR1L=0;
	//T1CON=0b0010.0001;  // Timer1 int, div8, run
	TMR1ON=1;
	while ((!TMP_1)&&(!TMR1IF))//&&(t_low)
	{
	};
	//T1CON=0b0010.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_low.low8 = TMR1L;
	t_low.high8 = TMR1H;
	
	TMR1H=0;
	TMR1L=0;
	//T1CON=0b0010.0001;  // Timer1 int, div8, run
	TMR1ON=1;
	while ((TMP_1)&&(!TMR1IF))//&&(t_high)
	{
	};
	//T1CON=0b0010.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_high.low8 = TMR1L;
	t_high.high8 = TMR1H;

	F_SensorG|=TMR1IF;
	
#pragma updateBank 1 /* ON, all cores */
}
// #######################################
//  Temp2 / Kuehlfach messen
// #######################################
void Measure_K(void)
{
	T1CON=0b0011.0000;  // Timer1 int, div8, stop
	TMR1H=0;
	TMR1L=0;
	TMR1IF=0;
	//T1CON=0b0011.0001;  // Timer1 int, div8, run
	TMR1ON=1;
#pragma updateBank 0 /* OFF, all cores */

	while ((!TMP_2)&&(!TMR1IF))//&&(t_low)
	{
	};
	//T1CON=0b0011.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_low.low8 = TMR1L;
	t_low.high8 = TMR1H;
	
	TMR1H=0;
	TMR1L=0;
	//T1CON=0b0011.0001;  // Timer1 int, div8, run
	TMR1ON=1;
	while ((TMP_2)&&(!TMR1IF))//&&(t_high)
	{
	};
	//T1CON=0b0010.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_high.low8 = TMR1L;
	t_high.high8 = TMR1H;
	
	
	TMR1H=0;
	TMR1L=0;
	//T1CON=0b0010.0001;  // Timer1 int, div8, run
	TMR1ON=1;
	while ((!TMP_2)&&(!TMR1IF))//&&(t_low)
	{
	};
	//T1CON=0b0010.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_low.low8 = TMR1L;
	t_low.high8 = TMR1H;
	
	TMR1H=0;
	TMR1L=0;
	//T1CON=0b0010.0001;  // Timer1 int, div8, run
	TMR1ON=1;
	while ((TMP_2)&&(!TMR1IF))//&&(t_high)
	{
	};
	//T1CON=0b0010.0000;  // Timer1 int, div8, stop
	TMR1ON=0;
	t_high.low8 = TMR1L;
	t_high.high8 = TMR1H;

	F_SensorK|=TMR1IF;
	
#pragma updateBank 1 /* ON, all cores */
}
// #######################################
//  Kompressor an und aus
// #######################################
void Komp_An(void)
{
#pragma updateBank 0 /* OFF, all cores */

	// Wenn keine Schutzzeit, einschalten
	if (!Komp_prtkt)
	{
		Komp_ON = 1;
		Komp_OnTime =0;
		F_MotorMax = 0;
		KOMP_RELAIS = 1;
	}
#pragma updateBank 1 /* ON, all cores */
}

void Komp_Aus(void)
{
#pragma updateBank 0 /* OFF, all cores */

	Komp_prtkt = KOMP_PROT;
	KOMP_RELAIS = 0;
	Komp_ON = 0;
	
#pragma updateBank 1 /* ON, all cores */
}

// #######################################
//  Kompressorschutz
// #######################################
void Komp_Control(void)
{
#pragma updateBank 0 /* OFF, all cores */

	// Kompressor soll gestartet werden
	if ((Komp_Req) && (!Komp_ON))
	{
		Komp_An();	
	}
	// Kompressor soll gestartet werden
	else if ((!Komp_Req) && (Komp_ON))
	{
		Komp_Aus();
	}
	
	// wenn Kompressor luft
	if (Komp_ON)
	{
		Komp_OnTime++;
		
		// Maximale Laufzeit berschritten
		if (Komp_OnTime > KOMP_MAXRUN)
		{
			Komp_Aus();
			F_MotorMax=1;
		}
	}
	else
	{
		// Wenn Kompressor aus, Schutzzeit zurueckzaehlen
		if (Komp_prtkt)
		{
			Komp_prtkt--;
		}
	}
#pragma updateBank 1 /* ON, all cores */
}


// #######################################
//  R E G L E R
// #######################################
void Regler(void)
{
#pragma updateBank 0 /* OFF, all cores */
	
	// Einschaltern
	if ((temp_G > G_TEMP_MAX) || (temp_K > K_TEMP_MAX))
	{
		Komp_Req = 1;	
	}
	// Ausschalten
	else if ((temp_G < G_TEMP_MIN) && (temp_K < K_TEMP_MIN))
	{
		Komp_Req = 0;	
	}
	
	// Ausschalten Frostschutz 
	if (temp_K < K_FROST_MIN)
	{
		Komp_Req = 0;
		F_Frost  = 1;
	}
	else
	{
		F_Frost = 0;
	}
	
#pragma updateBank 1 /* ON, all cores */	
}

// #######################################
//  Temperatur aus t_hig, t_low berechnen
// #######################################	
void Berechnen(void)
{
#pragma updateBank 0 /* OFF, all cores */

	// Alle Temperaturen in 0.2Grad Aufloesung
	
	// t_high * 400 
	mul16(t_high,2000);
	//  div by t_low
	divide();
	
	t_low.high8=q[1];
	t_low.low8=q[0];
	
	// Temperatur = 235-(...) 
	Temperatur = (int16)1160-(int16)t_low; // Temperatur enhlt jetzt die Temperatur in 0.2 Grad steps + sign
	
#pragma updateBank 1 /* ON, all cores */
}

// ##################################
//  Temperatur nach ASCII
// ##################################	
void Format_Temp(void)
{
#pragma updateBank 0 /* OFF, all cores */

	// Temperatur in ASCII + Vorzeichen
	if (VZ_merker)
	{ 
		bin2dec999(-Temperatur);
		Text[0]='-';
	}
	else
	{
		bin2dec999(Temperatur);
		Text[0]='+';
	}
	
#pragma updateBank 1 /* ON, all cores */	
}

// ##################################
//  Temperatur plausibilisieren
// ##################################	
void Check_Temp(void)
{
#pragma updateBank 0 /* OFF, all cores */

	if (Temperatur > K_CHECK_MAX )
		VZ_merker = 1;
	else if (Temperatur < K_CHECK_MIN)
		VZ_merker = 1;
	else
		VZ_merker = 0;
		
		
#pragma updateBank 1 /* ON, all cores */
}

// ##################################
//  Text fr Sensorfehler setzen
// ##################################	
void ErrorText(void)
{
#asm  
			;	Text[0,1,2,5] = '-';
	movlw .45
	movwf Text
	movwf Text+1
	movwf Text+2
	movwf Text+4
			;	Text[3] = '.';
	movlw .46
	movwf Text+3
			;	Text[5] = 'E';	
	movlw .69
	movwf Text+5
#endasm
	LED_R = 0;
}

/*------------------------------------------------------------------*/
/*     M     M    A    II  N   N                                    */
/*     MM   MM   A A   II  NN  N                                    */
/*     M M M M  A   A  II  N N N                                    */
/*     M  M  M  AAAAA  II  N  NN                                    */
/*     M     M  A   A  II  N   N                                    */
/*------------------------------------------------------------------*/

void main (void)
{
	uns8 ticks;

	init();
	
	LCDInit();	  // LCD Initialisieren
		
	LCDCls();	 // Bildschirm lschen

// und ausgeben
	LCDPos(0,0);         
	LCDString("DTemp1.4");
	LCDPos(1,0);
	LCDString("C.Schirp");
	
	Delay1ms(10000);
	

#pragma updateBank 0 /* OFF, all cores */
  
	while (1)
	{
		
		// todo
		// LCD schalten und initialisieren...?
		// Filterzeit verlngern
		
		LED_G = 0;	// Messungsanzeige an
		LED_R = 1;	// Fehleranzeige aus
		
		if (Blink)
			Text[7]=1;
		else if (F_MotorMax)
			Text[7]=3;
		else
			Text[7]=2;
		
		Blink ^=1;	// Blinker weiterzaehlen
		
		// ##################################
		// Gefrierfach Messung durchfhren
		// ##################################		
		Measure_G();
		Berechnen();
		med_a = Temperatur;
		
		Measure_G();
		Berechnen();
		med_b = Temperatur;
		
		Measure_G();
		Berechnen();
		med_c = Temperatur;		
		
		MedianFilter();

		Check_Temp();
		
		F_SensorG |= VZ_merker; // F_SensorX wird in Measure_X zurueckgesetzt

		 // Fehler?
		if (F_SensorG)
		{
			ErrorText();
			temp_G = K_FAIL_TEMP_G; // Default-Temp fuer Komp. aus
		}
		else
		{
			Text[5] = ' '; // ggfs. Fehlerausgabe loeschen
			
			// todo: Wenn fehler nicht regeln sondern Vertrauenszhler nutzen
			//
			
			// Filter (1xalt + 1xneu)
			temp_G+=Temperatur;
			
			// Vorzeichen merken
			if (temp_G < 0)
			{
				VZ_merker = 1;
			}
			else
			{
				VZ_merker = 0;
			}
			
			temp_G >>= 1; // Filter wieder normieren
			
			if (VZ_merker)
				temp_G |= (uns16)0x8000;
			
			Temperatur=temp_G<<1;	// von 0.2 Grad auf 0.1 Grad normieren, bi2dec liefert dann xx.x
			
			// nach ASCII wandlen
			Format_Temp();
		}
		// und ausgeben
		LCDPos(1,0);         
		LCDString(Text);
		
		// ##################################
		//  Khlfach Messung durchfhren
		// ##################################		
		Measure_K();
		Berechnen();
		med_a = Temperatur;
		
		Measure_K();
		Berechnen();
		med_b = Temperatur;
		
		Measure_K();
		Berechnen();
		med_c = Temperatur;
		
		Check_Temp();
		
		F_SensorK |= VZ_merker;	// F_SensorX wird in Measure_X zurueckgesetzt
		
		 // Fehler?
		if (F_SensorK)
		{
			ErrorText();
			temp_K = K_FAIL_TEMP_K; // Default-Temp fuer Komp. aus
		}
		else
		{
			Text[5] = ' '; // ggfs. Fehlerausgabe loeschen
			

			// Filter (1xalt + 1xneu)
			temp_K+=Temperatur;
			
			// Vorzeichen merken
			if (temp_K < 0)
			{
				VZ_merker = 1;
			}
			else
			{
				VZ_merker = 0;
			}
			
			temp_K >>= 1; // Filter wieder normieren
			
			if (VZ_merker)
				temp_K |= (uns16)0x8000;
			
			Temperatur=temp_K<<1;	// von 0.2 Grad auf 0.1 Grad normieren, bi2dec liefert dann xx.x
			
			// nach ASCII wandlen
			Format_Temp();
			
		}
		
		// Regelalgo ausfhren	
		Regler();
			
		// Und Kompressor bedienen
		Komp_Control();
		
		if (Komp_ON)
			Text[7]='K';
		else
			Text[7]=' ';
			
		// und ausgeben
		LCDPos(0,0);         
		LCDString(Text);
		
		// Fehlermerker resetten
		F_SensorK=0;
		F_SensorG=0;
		
		LED_G = 1;	// Messungsanzeige aus
		
		Delay1ms(10000);	// Messtakt: 10s


	}; 
#pragma updateBank 1 /* ON, all cores */


}